#include "V_D_UC.h"
#include "da.h"

#ifdef USE_agent

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* CALL:
*   DAmsg* daMsgGet(n,m)
*   	int   n;	-- Max size of message
*   	char* m;	-- Description of message
*
* DESCRIPTION:
*
* Return an unused DA message of at least n data bytes.   We  fill  in
* some data fields; the caller is responsible for the rest.
*
* Failure  can  occur  if  there's  no  more  space  available, and is
* indicated by a null value.
*
* AUTHOR: John Chambers.
*
* CHANGES:
*/
FCT	DAmsg* daMsgGet(n,m)
	int   n;	/* Max size of message */
	char* m;	/* Description of message */
{	DAmsg* v = 0;
:f	Fenter("daMsgGet");
:5	V5D "Called for n=%d %s",n,Dspp(m) D;
	if (!(r = (DAmsg*)GetChunk(n+sizeof(DAmsg),m))) Fail;
	r->magic = 0x12345678;	/* Magic number shows byte order */
	r->size  = n;
:f	Fexit;
	return v;
}

/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
* CALL:
*   daMsgFree(p)
*   	DAmsg* p;
*
* DESCRIPTION:
*
* Free a DA message allocated above. Actually, we can handle any block
* that  was  delivered by the memchunk package.  We don't even bomb if
* the caller sends us a null pointer.
*
* The return value is currently always 1.   We  might  do  some  error
* checking at some time in the future, and if we do, we may add a zero
* return.  In any case, callers normally ignore our return value.
*
* AUTHOR: John Chambers.
*
* CHANGES:
*/
FCT daMsgFree(p)
	DAmsg* p;
{
:f	Fenter("daMsgFree");
:5	V5D "Called for %08lX.",p D;
	IRelChunk(p);
:f	Fexit;
	return 1;
}
#endif
